"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.processTransactionResponse = void 0;
const ethers_1 = require("./ethers");
const request_1 = require("./request");
const routes_json_1 = __importDefault(require("./routes.json"));
const registerTransaction = async (twoPi, vaultIdentifier, receipt) => {
    if (twoPi.address) {
        const path = routes_json_1.default.transactionsPath.replace(/{wallet_id}/g, twoPi.address);
        const data = {
            transaction: {
                id: receipt.transactionHash,
                contract_identifier: vaultIdentifier
            }
        };
        try {
            await (0, request_1.post)(twoPi, path, data);
            return receipt;
        }
        catch (error) {
            console.error(error);
        }
    }
};
const processTransaction = async (twoPi, signer, vaultIdentifier, transaction, description, receipts) => {
    if (signer) {
        try {
            const response = await signer.sendTransaction(transaction);
            const receipt = await response.wait();
            const registered = await registerTransaction(twoPi, vaultIdentifier, receipt);
            receipts.push(receipt);
            // Status === 1 means success, 0 means it was rejected
            if (!registered?.status) {
                const hash = receipt.transactionHash;
                const message = `${description} transaction failed (${hash})`;
                return { status: 'error', cursor: receipts.length, message, receipts };
            }
        }
        catch (error) {
            const { reason } = Object(error);
            const message = reason || 'Unknown error';
            return { status: 'failed', cursor: receipts.length, message, receipts };
        }
    }
};
const registerTransactionProposals = async (twoPi, { provider, transactions }) => {
    for (const transactionData of transactions) {
        const { description, transaction: data, transaction_details: details } = transactionData;
        const proposal = { transaction: { data, description, details, provider } };
        await (0, request_1.post)(twoPi, routes_json_1.default.proposalsPath, { proposal });
    }
};
const processTransactionResponse = async (twoPi, vaultIdentifier, response) => {
    const receipts = [];
    if (response.status === 200) {
        for (const data of response.data.data) {
            await registerTransactionProposals(twoPi, data);
            const signer = (0, ethers_1.getSigner)(twoPi, {
                chainId: data.provider.chain_id,
                rpcUrl: data.provider.rpc_url
            });
            for (const transactionData of data.transactions) {
                const result = await processTransaction(twoPi, signer, vaultIdentifier, transactionData.transaction, transactionData.description, receipts);
                if (result) {
                    return result;
                }
            }
        }
        return { status: 'success', receipts };
    }
    else {
        return {
            status: 'error',
            message: `Withdraw response status was ${response.status} (and 200 was expected)`
        };
    }
};
exports.processTransactionResponse = processTransactionResponse;
