"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const buffer_1 = require("buffer");
const child_process_1 = require("child_process");
const copyFiles = (source, destination) => {
    const stats = fs_1.default.statSync(source);
    if (stats.isDirectory()) {
        fs_1.default.mkdirSync(destination, { recursive: true });
        fs_1.default.readdirSync(source).forEach(child => {
            copyFiles(path_1.default.join(source, child), path_1.default.join(destination, child));
        });
    }
    else {
        fs_1.default.copyFileSync(source, destination);
    }
};
const createEnv = (appPath, options) => {
    const filePath = path_1.default.join(appPath, '.env');
    let buffer = buffer_1.Buffer.from('');
    Object.entries(options).forEach(([name, value]) => {
        const constantName = name.replace(/([A-Z])/g, '_$1').toUpperCase();
        const line = buffer_1.Buffer.from(`${constantName}="${value}"${os_1.default.EOL}`);
        buffer = buffer_1.Buffer.concat([buffer, line]);
    });
    fs_1.default.writeFileSync(filePath, buffer);
};
const packageJsonData = (appName) => ({
    name: appName,
    private: true,
    scripts: {
        test: 'echo "Error: no test specified" && exit 1'
    }
});
const createPackageJson = (appPath, appName) => {
    const filePath = path_1.default.join(appPath, 'package.json');
    const jsonString = JSON.stringify(packageJsonData(appName), null, 2);
    fs_1.default.writeFileSync(filePath, `${jsonString}${os_1.default.EOL}`);
};
const installDependencies = (appPath) => {
    const deps = ['axios', 'dotenv', 'ethers'];
    const { status, stdout, stderr } = (0, child_process_1.spawnSync)('npm', ['install', ...deps], { cwd: appPath });
    if (status === 0) {
        console.log(stdout.toString());
    }
    else {
        console.error(stderr.toString());
        process.exit(status || 1);
    }
};
const tryGitInit = (appPath) => {
    const options = { cwd: appPath };
    try {
        (0, child_process_1.execSync)('git --version', options);
        (0, child_process_1.execSync)('git init', options);
        return true;
    }
    catch (_error) {
        return false;
    }
};
const createApp = (appPath, options) => {
    const appName = path_1.default.basename(appPath);
    if (fs_1.default.existsSync(appPath)) {
        console.log(`The supplied path (${appPath}) exists, please provide a brand new one`);
        process.exit(1);
    }
    console.log('Copying template files...');
    copyFiles(path_1.default.join(__dirname, 'templates', 'javascript'), appPath);
    console.log('Saving credentials...');
    createEnv(appPath, options);
    console.log('Creating initial package.json...');
    createPackageJson(appPath, appName);
    console.log('Installing dependencies...');
    installDependencies(appPath);
    console.log('Trying to initialize a git repository...');
    tryGitInit(appPath);
};
exports.default = createApp;
